--|> CORE
-------------------------------------------------------------------------------
local _, NS = ...

--> LIBS
local RC = LibStub("LibRangeCheck-2.0")

--> Upvalues
local GetFactionColor = GetFactionColor
local UnitIsUnit = UnitIsUnit
local UnitIsPlayer = UnitIsPlayer
local UnitExists = UnitExists
local UnitGUID = UnitGUID
local UnitFactionGroup = UnitFactionGroup
local C_Timer_After = C_Timer.After
local C_NamePlate_GetNamePlateForUnit = C_NamePlate.GetNamePlateForUnit

--> Settings
local FFNameplateShown = false
local FTFUnitGUID = ""

--> Print Addon Message
-------------------------------------------------------------------------------
function NS.PrintAddonMessage(msg)
    if not msg then
        return
    end
    (SELECTED_CHAT_FRAME or DEFAULT_CHAT_FRAME):AddMessage(
        "|T" .. NS.Constants.Strings.mediaPath .. "Icons\\chat.tga:0|t  " .. msg
    )
end

--> Event Frame
---------------------------------------
local EFrame = CreateFrame("Frame", nil, UIParent)
EFrame:Hide()

--> Is unit valid for markers?
---------------------------------------
local function IsUnitValid(unit)
    if UnitExists(unit) and UnitIsPlayer(unit) then
        return true
    else
        return false
    end
end

--> Hide Focus
---------------------------------------
local function HideFocus()
    NS.FF:ClearAllPoints()
    NS.FF:Hide()
    NS.FF.RangeText:SetText("")
    NS.FF.ag:Stop()
end

--> Show Focus
---------------------------------------
local function ShowFocus()
    if C_NamePlate_GetNamePlateForUnit("focus") then
        FFNameplateShown = true
        NS.FF:SetPoint("BOTTOM", C_NamePlate_GetNamePlateForUnit("focus"), "CENTER", 0, 20)
        if not NS.FF.ag:IsPlaying() then
            NS.FF.ag:Play()
        end
        NS.RangeChecker()
    else
        HideFocus()
    end
end

--> Hide Focus Target
---------------------------------------
local function HideFocusTarget()
    NS.FTF:ClearAllPoints()
    NS.FTF:Hide()
    FTFUnitGUID = ""
    NS.FTF.ag:Stop()
end

--> Show Focus Target
local function ShowFocusTarget()
    if UnitIsUnit("focustarget", "focus") then
        HideFocusTarget()
        return
    end
    if C_NamePlate_GetNamePlateForUnit("focustarget") then
        NS.FTF:SetPoint("BOTTOM", C_NamePlate_GetNamePlateForUnit("focustarget"), "CENTER", 0, 40)
        if FTFUnitGUID ~= UnitGUID("focustarget") then
            FTFUnitGUID = UnitGUID("focustarget")
            if not NS.FTF.ag:IsPlaying() then
                NS.FTF.ag:Play()
            end
        end
    else
        HideFocusTarget()
    end
end

--> Enable
---------------------------------------
function NS.Enable()
    ShowFocus()
    ShowFocusTarget()
    EFrame:RegisterEvent("NAME_PLATE_UNIT_ADDED")
    EFrame:RegisterEvent("NAME_PLATE_UNIT_REMOVED")
    EFrame:RegisterEvent("PLAYER_FOCUS_CHANGED")
    EFrame:RegisterEvent("UNIT_TARGET")
end

--> Disalbe
---------------------------------------
function NS.Disable()
    FFNameplateShown = false
    HideFocus()
    HideFocusTarget()
    EFrame:UnregisterEvent("NAME_PLATE_UNIT_ADDED")
    EFrame:UnregisterEvent("NAME_PLATE_UNIT_REMOVED")
    EFrame:UnregisterEvent("PLAYER_FOCUS_CHANGED")
    EFrame:UnregisterEvent("UNIT_TARGET")
end

--> Event Handler
---------------------------------------
EFrame:RegisterEvent("PLAYER_ENTERING_WORLD")
if FTLCharOptions.Enabled then
    NS.Enable()
end
EFrame:SetScript(
    "OnEvent",
    function(self, event, ...)
        return self[event] and self[event](self, ...)
    end
)

--|> EVENTS
-------------------------------------------------------------------------------
--> PLAYER_ENTERING_WORLD
local enteredWorld = true
function EFrame.PLAYER_ENTERING_WORLD()
    if enteredWorld then
        NS.PrintAddonMessage(NS.Constants.Strings.loaded)
        enteredWorld = nil
    end
end

--> NAME_PLATE_UNIT_ADDED
-------------------------------------------------------------------------------
function EFrame.NAME_PLATE_UNIT_ADDED(_, unit)
    if not IsUnitValid(unit) then
        return
    end
    if UnitIsUnit("focus", unit) then --: focus plate appears
        ShowFocus()
    elseif UnitIsUnit("focustarget", unit) then --: focus target plate appears
        ShowFocusTarget()
    end
end

--> NAME_PLATE_UNIT_REMOVED
-------------------------------------------------------------------------------
function EFrame.NAME_PLATE_UNIT_REMOVED(_, unit)
    if not IsUnitValid(unit) then
        return
    end
    if UnitIsUnit("focus", unit) then --: focus plate removed
        HideFocus()
    elseif UnitIsUnit("focustarget", unit) then --: focus target plate removed
        HideFocusTarget()
    end
end

--> PLAYER_FOCUS_CHANGED
-------------------------------------------------------------------------------
local focusFaction
local fColor
function EFrame.PLAYER_FOCUS_CHANGED()
    if not IsUnitValid("focus") then --> valid check
        HideFocus()
        HideFocusTarget()
        return
    else
        focusFaction = UnitFactionGroup("focus")
        if focusFaction == "Horde" or focusFaction == "Alliance" then
            NS.FF.faction:SetTexture("Interface/Timer/" .. focusFaction .. "-Logo")
            fColor = GetFactionColor(focusFaction)
            NS.FF.color:SetVertexColor(fColor.r, fColor.g, fColor.b)
        else
            NS.FF.faction:SetTexture("Interface/Common/spacer")
            NS.FF.color:SetVertexColor(0.5, 0.5, 0.5)
        end
        ShowFocus()
        if UnitExists("focustarget") then
            ShowFocusTarget()
        end
    end
    focusFaction = nil
    fColor = nil
end

--> UNIT_TARGET
-------------------------------------------------------------------------------
--: Check for changes to our focus's target
function EFrame.UNIT_TARGET(_, unit)
    if not IsUnitValid(unit) then
        return
    end
    if UnitIsUnit("focus", unit) and not UnitIsUnit("focus", "focustarget") then -- focus changed target
        NS.FF.RangeText:SetText("")
        if UnitExists("focustarget") then
            ShowFocusTarget()
        else
            HideFocusTarget()
            FTFUnitGUID = ""
        end
    elseif not UnitExists("focustarget") then
        HideFocusTarget()
        FTFUnitGUID = ""
    end
end

--> ESTIMATE UNIT RANGE
-------------------------------------------------------------------------------
local function EstimateUnitRange(unit)
    local minRange, maxRange = RC:GetRange(unit)
    if not minRange then
        return "--"
    elseif not maxRange then
        return minRange, "+"
    elseif minRange == 0 then
        return ""
    else
        return minRange
    end
end

-->  Get estimated distance
-------------------------------------------------------------------------------
local range, txt
local function GetDistance()
    range, txt = EstimateUnitRange("focus")
    if range then
        if txt ~= nil then
            range = range .. txt
        end
        return range
    end
end

--> RANGE UPDATER
-------------------------------------------------------------------------------
function NS.RangeChecker()
    if not FFNameplateShown then
        NS.FF.RangeText:SetText("")
        return
    else
        --> Focus exists and nameplate shown
        NS.FF.RangeText:SetText(GetDistance())
        C_Timer_After(
            0.3,
            function()
                NS.RangeChecker()
            end
        )
    end
end

C_Timer_After(
    0.5,
    function()
        NS.RangeChecker()
    end
)
